@@ -161,8 +161,15 @@ class Cluster(object):
 
     compression = True
     """
-    Whether or not compression should be enabled when possible. Defaults to
-    :const:`True` and attempts to use snappy compression.
+    Controls compression for communications between the driver and Cassandra.
+    If left as the default of :const:`True`, either lz4 or snappy compression
+    may be used, depending on what is supported by both the driver
+    and Cassandra.  If both are fully supported, lz4 will be preferred.
+
+    You may also set this to 'snappy' or 'lz4' to request that specific
+    compression type.
+
+    Setting this to :const:`False` disables compression.
     """
 
     auth_provider = None
@@ -579,11 +586,8 @@ class Cluster(object):
             for listener in self.listeners:
                 listener.on_up(host)
         finally:
-            host._handle_node_up_condition.acquire()
-            if host._currently_handling_node_up:
+            with host.lock:
                 host._currently_handling_node_up = False
-                host._handle_node_up_condition.notify()
-            host._handle_node_up_condition.release()
 
         # see if there are any pools to add or remove now that the host is marked up
         for session in self.sessions:
@@ -596,18 +600,23 @@ class Cluster(object):
         if self._is_shutdown:
             return
 
-        host._handle_node_up_condition.acquire()
-        while host._currently_handling_node_up:
-            host._handle_node_up_condition.wait()
-        host.handling_up_down = True
-        host._handle_node_up_condition.release()
+        log.debug("Waiting to acquire lock for handling up status of node %s", host)
+        with host.lock:
+            if host._currently_handling_node_up:
+                log.debug("Another thread is already handling up status of node %s", host)
+                return
 
-        if host.is_up:
-            return
+            if host.is_up:
+                log.debug("Host %s was already marked up", host)
+                return
 
+            host._currently_handling_node_up = True
+        log.debug("Starting to handle up status of node %s", host)
+
+        have_future = False
         futures = set()
         try:
-            log.info("Host %s has been marked up", host)
+            log.info("Host %s may be up; will prepare queries and open connection pool", host)
 
             reconnector = host.get_and_set_reconnection_handler(None)
             if reconnector:
@@ -615,20 +624,25 @@ class Cluster(object):
                 reconnector.cancel()
 
             self._prepare_all_queries(host)
-            log.debug("Done preparing all queries for host %s", host)
+            log.debug("Done preparing all queries for host %s, ", host)
 
             for session in self.sessions:
                 session.remove_pool(host)
 
+            log.debug("Signalling to load balancing policy that host %s is up", host)
             self.load_balancing_policy.on_up(host)
+
+            log.debug("Signalling to control connection that host %s is up", host)
             self.control_connection.on_up(host)
 
+            log.debug("Attempting to open new connection pools for host %s", host)
             futures_lock = Lock()
             futures_results = []
             callback = partial(self._on_up_future_completed, host, futures, futures_results, futures_lock)
             for session in self.sessions:
                 future = session.add_or_renew_pool(host, is_host_addition=False)
                 if future is not None:
+                    have_future = True
                     future.add_done_callback(callback)
                     futures.add(future)
         except Exception:
@@ -638,11 +652,13 @@ class Cluster(object):
 
             self._cleanup_failed_on_up_handling(host)
 
-            host._handle_node_up_condition.acquire()
-            host._currently_handling_node_up = False
-            host._handle_node_up_condition.notify()
-            host._handle_node_up_condition.release()
+            with host.lock:
+                host._currently_handling_node_up = False
             raise
+        else:
+            if not have_future:
+                with host.lock:
+                    host._currently_handling_node_up = False
 
         # for testing purposes
         return futures
@@ -665,11 +681,11 @@ class Cluster(object):
             log.debug("Old host reconnector found for %s, cancelling", host)
             old_reconnector.cancel()
 
-        log.debug("Staring reconnector for host %s", host)
+        log.debug("Starting reconnector for host %s", host)
         reconnector.start()
 
     @run_in_executor
-    def on_down(self, host, is_host_addition, force_if_down=False):
+    def on_down(self, host, is_host_addition, expect_host_to_be_down=False):
         """
         Intended for internal use only.
         """
@@ -677,7 +693,7 @@ class Cluster(object):
             return
 
         with host.lock:
-            if (not (host.is_up or force_if_down)) or host.is_currently_reconnecting():
+            if (not host.is_up and not expect_host_to_be_down) or host.is_currently_reconnecting():
                 return
 
             host.set_down()
@@ -765,11 +781,12 @@ class Cluster(object):
             session.on_remove(host)
         for listener in self.listeners:
             listener.on_remove(host)
+        self.control_connection.on_remove(host)
 
-    def signal_connection_failure(self, host, connection_exc, is_host_addition):
+    def signal_connection_failure(self, host, connection_exc, is_host_addition, expect_host_to_be_down=False):
         is_down = host.signal_connection_failure(connection_exc)
         if is_down:
-            self.on_down(host, is_host_addition, force_if_down=True)
+            self.on_down(host, is_host_addition, expect_host_to_be_down)
         return is_down
 
     def add_host(self, address, datacenter=None, rack=None, signal=True):
@@ -1200,8 +1217,12 @@ class Session(object):
                 self.cluster.signal_connection_failure(host, conn_exc, is_host_addition)
                 return False
             except Exception as conn_exc:
-                log.warn("Failed to create connection pool for new host %s: %s", host, conn_exc)
-                self.cluster.signal_connection_failure(host, conn_exc, is_host_addition)
+                log.warn("Failed to create connection pool for new host %s: %s",
+                         host, conn_exc)
+                # the host itself will still be marked down, so we need to pass
+                # a special flag to make sure the reconnector is created
+                self.cluster.signal_connection_failure(
+                        host, conn_exc, is_host_addition, expect_host_to_be_down=True)
                 return False
 
             previous = self._pools.get(host)
@@ -1658,10 +1679,10 @@ class ControlConnection(object):
         if change_type == "UP":
             if host is None:
                 # this is the first time we've seen the node
-                self._cluster.scheduler.schedule(1, self.refresh_node_list_and_token_map)
+                self._cluster.scheduler.schedule(2, self.refresh_node_list_and_token_map)
             else:
                 # this will be run by the scheduler
-                self._cluster.scheduler.schedule(1, self._cluster.on_up, host)
+                self._cluster.scheduler.schedule(2, self._cluster.on_up, host)
         elif change_type == "DOWN":
             # Note that there is a slight risk we can receive the event late and thus
             # mark the host down even though we already had reconnected successfully.
