@@ -31,23 +31,15 @@ from cassandra.decoder import (ReadyMessage, AuthenticateMessage, OptionsMessage
                                StartupMessage, ErrorMessage, CredentialsMessage,
                                QueryMessage, ResultMessage, decode_response,
                                InvalidRequestException, SupportedMessage)
+from cassandra.util import OrderedDict
 
 
 log = logging.getLogger(__name__)
 
-locally_supported_compressions = {}
-
-try:
-    import snappy
-except ImportError:
-    pass
-else:
-    # work around apparently buggy snappy decompress
-    def decompress(byts):
-        if byts == '\x00':
-            return ''
-        return snappy.decompress(byts)
-    locally_supported_compressions['snappy'] = (snappy.compress, decompress)
+# We use an ordered dictionary and specifically add lz4 before
+# snappy so that lz4 will be preferred. Changing the order of this
+# will change the compression preferences for the driver.
+locally_supported_compressions = OrderedDict()
 
 try:
     import lz4
@@ -69,6 +61,18 @@ else:
 
     locally_supported_compressions['lz4'] = (lz4_compress, lz4_decompress)
 
+try:
+    import snappy
+except ImportError:
+    pass
+else:
+    # work around apparently buggy snappy decompress
+    def decompress(byts):
+        if byts == '\x00':
+            return ''
+        return snappy.decompress(byts)
+    locally_supported_compressions['snappy'] = (snappy.compress, decompress)
+
 
 MAX_STREAM_PER_CONNECTION = 127
 
@@ -302,7 +306,7 @@ class Connection(object):
             response = decode_response(stream_id, flags, opcode, body, self.decompressor)
         except Exception as exc:
             log.exception("Error decoding response from Cassandra. "
-                          "opcode: %04x; message contents: %r", opcode, body)
+                          "opcode: %04x; message contents: %r", opcode, msg)
             if callback is not None:
                 callback(exc)
             self.defunct(exc)
@@ -369,7 +373,22 @@ class Connection(object):
                           locally_supported_compressions.keys(),
                           remote_supported_compressions)
             else:
-                compression_type = iter(overlap).next()  # choose any
+                compression_type = None
+                if isinstance(self.compression, basestring):
+                    # the user picked a specific compression type ('snappy' or 'lz4')
+                    if self.compression not in remote_supported_compressions:
+                        raise ProtocolError(
+                            "The requested compression type (%s) is not supported by the Cassandra server at %s"
+                            % (self.compression, self.host))
+                    compression_type = self.compression
+                else:
+                    # our locally supported compressions are ordered to prefer
+                    # lz4, if available
+                    for k in locally_supported_compressions.keys():
+                        if k in overlap:
+                            compression_type = k
+                            break
+
                 # set the decompressor here, but set the compressor only after
                 # a successful Ready message
                 self._compressor, self.decompressor = \
@@ -415,6 +434,9 @@ class Connection(object):
                 raise ConnectionException(
                     "Failed to initialize new connection to %s: %s"
                     % (self.host, startup_response.summary_msg()))
+        elif isinstance(startup_response, ConnectionShutdown):
+            log.debug("Connection to %s was closed during the startup handshake", (self.host))
+            raise startup_response
         else:
             msg = "Unexpected response during Connection setup: %r"
             log.error(msg, startup_response)
