@@ -1,3 +1,17 @@
+# Copyright 2013-2014 DataStax, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from binascii import hexlify
 import calendar
 from collections import namedtuple
@@ -533,11 +547,11 @@ class ResultMessage(_MessageType):
                                     " entire result set." % (optid,))
         if typeclass in (ListType, SetType):
             subtype = cls.read_type(f)
-            typeclass = typeclass.apply_parameters(subtype)
+            typeclass = typeclass.apply_parameters((subtype,))
         elif typeclass == MapType:
             keysubtype = cls.read_type(f)
             valsubtype = cls.read_type(f)
-            typeclass = typeclass.apply_parameters(keysubtype, valsubtype)
+            typeclass = typeclass.apply_parameters((keysubtype, valsubtype))
         elif typeclass == CUSTOM_TYPE:
             classname = read_string(f)
             typeclass = lookup_casstype(classname)
