@@ -69,7 +69,6 @@ class Host(object):
     lock = None
 
     _currently_handling_node_up = False
-    _handle_node_up_condition = None
 
     def __init__(self, inet_address, conviction_policy_factory, datacenter=None, rack=None):
         if inet_address is None:
@@ -81,7 +80,6 @@ class Host(object):
         self.conviction_policy = conviction_policy_factory(self)
         self.set_location_info(datacenter, rack)
         self.lock = RLock()
-        self._handle_node_up_condition = Condition()
 
     @property
     def datacenter(self):
@@ -155,11 +153,9 @@ class _ReconnectionHandler(object):
 
     def start(self):
         if self._cancelled:
+            log.debug("Reconnection handler was cancelled before starting")
             return
 
-        # TODO cancel previous reconnection handlers? That's probably the job
-        # of whatever created this.
-
         first_delay = self.schedule.next()
         self.scheduler.schedule(first_delay, self.run)
 
@@ -171,9 +167,20 @@ class _ReconnectionHandler(object):
         try:
             conn = self.try_reconnect()
         except Exception as exc:
-            next_delay = self.schedule.next()
+            try:
+                next_delay = self.schedule.next()
+            except StopIteration:
+                # the schedule has been exhausted
+                next_delay = None
+
+            # call on_exception for logging purposes even if next_delay is None
             if self.on_exception(exc, next_delay):
-                self.scheduler.schedule(next_delay, self.run)
+                if next_delay is None:
+                    log.warn(
+                        "Will not continue to retry reconnection attempts "
+                        "due to an exhausted retry schedule")
+                else:
+                    self.scheduler.schedule(next_delay, self.run)
         else:
             if not self._cancelled:
                 self.on_reconnection(conn)
