@@ -1,3 +1,17 @@
+# Copyright 2013-2014 DataStax, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+# http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """
 This module holds classes for working with prepared statements and
 specifying consistency levels and retry policies for individual
@@ -214,10 +228,47 @@ class BoundStatement(Statement):
     def bind(self, values):
         """
         Binds a sequence of values for the prepared statement parameters
-        and returns this instance.  Note that `values` *must* be a
-        sequence, even if you are only binding one value.
+        and returns this instance.  Note that `values` *must* be:
+        * a sequence, even if you are only binding one value, or
+        * a dict that relates 1-to-1 between dict keys and columns
         """
         col_meta = self.prepared_statement.column_metadata
+
+        # special case for binding dicts
+        if isinstance(values, dict):
+            dict_values = values
+            values = []
+
+            # sort values accordingly
+            for col in col_meta:
+                try:
+                    values.append(dict_values[col[2]])
+                except KeyError:
+                    raise KeyError(
+                        'Column name `%s` not found in bound dict.' %
+                        (col[2]))
+
+            # ensure a 1-to-1 dict keys to columns relationship
+            if len(dict_values) != len(col_meta):
+                # find expected columns
+                columns = set()
+                for col in col_meta:
+                    columns.add(col[2])
+
+                # generate error message
+                if len(dict_values) > len(col_meta):
+                    difference = set(dict_values.keys()).difference(columns)
+                    msg = "Too many arguments provided to bind() (got %d, expected %d). " + \
+                          "Unexpected keys %s."
+                else:
+                    difference = set(columns).difference(dict_values.keys())
+                    msg = "Too few arguments provided to bind() (got %d, expected %d). " + \
+                          "Expected keys %s."
+
+                # exit with error message
+                msg = msg % (len(values), len(col_meta), difference)
+                raise ValueError(msg)
+
         if len(values) > len(col_meta):
             raise ValueError(
                 "Too many arguments provided to bind() (got %d, expected %d)" %
@@ -390,8 +441,10 @@ class QueryTrace(object):
         while True:
             time_spent = time.time() - start
             if max_wait is not None and time_spent >= max_wait:
-                raise TraceUnavailable("Trace information was not available within %f seconds" % (max_wait,))
+                raise TraceUnavailable(
+                    "Trace information was not available within %f seconds. Consider raising Session.max_trace_wait." % (max_wait,))
 
+            log.debug("Attempting to fetch trace info for trace ID: %s", self.trace_id)
             session_results = self._execute(
                 self._SELECT_SESSIONS_FORMAT, (self.trace_id,), time_spent, max_wait)
 
@@ -399,6 +452,7 @@ class QueryTrace(object):
                 time.sleep(self._BASE_RETRY_SLEEP * (2 ** attempt))
                 attempt += 1
                 continue
+            log.debug("Fetched trace info for trace ID: %s", self.trace_id)
 
             session_row = session_results[0]
             self.request_type = session_row.request
@@ -407,9 +461,11 @@ class QueryTrace(object):
             self.coordinator = session_row.coordinator
             self.parameters = session_row.parameters
 
+            log.debug("Attempting to fetch trace events for trace ID: %s", self.trace_id)
             time_spent = time.time() - start
             event_results = self._execute(
                 self._SELECT_EVENTS_FORMAT, (self.trace_id,), time_spent, max_wait)
+            log.debug("Fetched trace events for trace ID: %s", self.trace_id)
             self.events = tuple(TraceEvent(r.activity, r.event_id, r.source, r.source_elapsed, r.thread)
                                 for r in event_results)
             break
